@@ -73,9 +73,10 @@ def fix_shebang(f, prefix, build_python, osx_is_app=False):
             bytes_ = True
             m = SHEBANG_PAT.match(mm)
 
-        python_str = b'python' if bytes_ else 'python'
+        python_pattern = (re.compile(br'\/python[w]?(?:$|\s|\Z)', re.M) if bytes_ else
+                          re.compile(r'\/python[w]?(:$|\s|\z)', re.M))
 
-        if not (m and python_str in m.group()):
+        if not (m and python_pattern.search(m.group())):
             return
 
         data = mm[:]
@@ -485,7 +486,11 @@ def check_overlinking(m, files):
         info_prelude = "   INFO ({},{})".format(pkg_name, f)
         msg_prelude = err_prelude if m.config.error_overlinking else warn_prelude
 
-        runpaths = get_runpaths(path)
+        try:
+            runpaths = get_runpaths(path)
+        except:
+            print_msg(errors, '{}: pyldd.py failed to process'.format(warn_prelude))
+            continue
         if len(runpaths):
             print_msg(errors, '{}: runpaths {} found in {}'.format(msg_prelude,
                                                                    runpaths,
