@@ -6,14 +6,15 @@
 
 from __future__ import absolute_import, division, print_function
 
-from collections import OrderedDict
+from collections import OrderedDict, defaultdict
 from locale import getpreferredencoding
 import os
 from os.path import isdir, isfile, abspath
 import random
 import re
-import subprocess
+import shutil
 import string
+import subprocess
 import sys
 import tarfile
 import tempfile
@@ -27,6 +28,7 @@ from .conda_interface import pkgs_dirs
 from .conda_interface import conda_43
 from .conda_interface import specs_from_url
 from .conda_interface import memoized
+from .conda_interface import VersionOrder
 
 from conda_build import exceptions, utils, environ
 from conda_build.metadata import MetaData, combine_top_level_metadata_with_output
@@ -37,8 +39,14 @@ from conda_build.exceptions import DependencyNeedsBuildingError
 from conda_build.index import get_build_index
 # from conda_build.jinja_context import pin_subpackage_against_outputs
 
+
+def odict_representer(dumper, data):
+    return dumper.represent_dict(data.items())
+
+
 yaml.add_representer(set, yaml.representer.SafeRepresenter.represent_list)
 yaml.add_representer(tuple, yaml.representer.SafeRepresenter.represent_list)
+yaml.add_representer(OrderedDict, odict_representer)
 
 
 def bldpkg_path(m):
@@ -147,7 +155,7 @@ def get_pin_from_build(m, dep, build_dep_versions):
     if (version and dep_name in m.config.variant.get('pin_run_as_build', {}) and
             not (dep_name == 'python' and (m.noarch or m.noarch_python)) and
             dep_name in build_dep_versions):
-        pin_cfg = m.config.variant['pin_run_as_build'][dep_name]
+        pin_cfg = m.config.variant['pin_run_as_build'][dep_name.replace('-', '_')]
         if isinstance(pin_cfg, str):
             # if pin arg is a single 'x.x', use the same value for min and max
             pin_cfg = dict(min_pin=pin_cfg, max_pin=pin_cfg)
@@ -173,18 +181,27 @@ def _filter_run_exports(specs, ignore_list):
     return filtered_specs
 
 
-def find_pkg_dir_or_file_in_pkgs_dirs(pkg_dist, m):
+def find_pkg_dir_or_file_in_pkgs_dirs(pkg_dist, m, files_only=False):
     _pkgs_dirs = pkgs_dirs + list(m.config.bldpkgs_dirs)
     pkg_loc = None
     for pkgs_dir in _pkgs_dirs:
         pkg_dir = os.path.join(pkgs_dir, pkg_dist)
         pkg_file = os.path.join(pkgs_dir, pkg_dist + '.tar.bz2')
-        if os.path.isdir(pkg_dir):
+        if not files_only and os.path.isdir(pkg_dir):
             pkg_loc = pkg_dir
             break
         elif os.path.isfile(pkg_file):
             pkg_loc = pkg_file
             break
+        elif files_only and os.path.isdir(pkg_dir):
+            pkg_loc = pkg_file
+            # create the tarball on demand.  This is so that testing on archives works.
+            with tarfile.open(pkg_file, 'w:bz2') as archive:
+                for entry in os.listdir(pkg_dir):
+                    archive.add(entry, arcname=os.path.relpath(entry, pkg_dir))
+            pkg_subdir = os.path.join(m.config.croot, m.config.host_subdir)
+            pkg_loc = os.path.join(pkg_subdir, os.path.basename(pkg_file))
+            shutil.move(pkg_file, pkg_loc)
     return pkg_loc
 
 
@@ -213,50 +230,55 @@ def _read_specs_from_package(pkg_loc, pkg_dist):
     return specs
 
 
-def get_upstream_pins(m, actions, env):
-    """Download packages from specs, then inspect each downloaded package for additional
-    downstream dependency specs.  Return these additional specs."""
-
-    env_specs = m.meta.get('requirements', {}).get(env, [])
-    explicit_specs = [req.split(' ')[0] for req in env_specs] if env_specs else []
-    linked_packages = actions.get('LINK', [])
-    linked_packages = [pkg for pkg in linked_packages if pkg.name in explicit_specs]
-
-    # edit the plan to download all necessary packages
-    for key in ('LINK', 'EXTRACT', 'UNLINK'):
-        if key in actions:
-            del actions[key]
-    # this should be just downloading packages.  We don't need to extract them -
-    #    we read contents directly
-
+def execute_download_actions(m, actions, env, package_subset=None, require_files=False):
     index, index_ts = get_build_index(getattr(m.config, '{}_subdir'.format(env)),
                                       bldpkgs_dir=m.config.bldpkgs_dir,
                                       output_folder=m.config.output_folder,
                                       channel_urls=m.config.channel_urls,
                                       debug=m.config.debug, verbose=m.config.verbose,
                                       locking=m.config.locking, timeout=m.config.timeout)
+
+    # this should be just downloading packages.  We don't need to extract them -
+    #    we read contents directly
     if 'FETCH' in actions or 'EXTRACT' in actions:
         # this is to force the download
         execute_actions(actions, index, verbose=m.config.debug)
-    ignore_list = utils.ensure_list(m.get_value('build/ignore_run_exports'))
 
-    additional_specs = {}
-    for pkg in linked_packages:
+    pkg_files = {}
+
+    packages = actions.get('LINK', [])
+    package_subset = utils.ensure_list(package_subset)
+    selected_packages = set()
+    if package_subset:
+        for pkg in package_subset:
+            if hasattr(pkg, 'name'):
+                if pkg in packages:
+                    selected_packages.add(pkg)
+            else:
+                pkg_name = pkg.split()[0]
+                for link_pkg in packages:
+                    if pkg_name == link_pkg.name:
+                        selected_packages.add(link_pkg)
+                        break
+        packages = selected_packages
+
+    for pkg in packages:
         if hasattr(pkg, 'dist_name'):
             pkg_dist = pkg.dist_name
         else:
             pkg = strip_channel(pkg)
             pkg_dist = pkg.split(' ')[0]
-        pkg_loc = find_pkg_dir_or_file_in_pkgs_dirs(pkg_dist, m)
+        pkg_loc = find_pkg_dir_or_file_in_pkgs_dirs(pkg_dist, m, files_only=require_files)
 
         # ran through all pkgs_dirs, and did not find package or folder.  Download it.
         # TODO: this is a vile hack reaching into conda's internals. Replace with
         #    proper conda API when available.
         if not pkg_loc and conda_43:
             try:
+                pkg_record = [_ for _ in index if _.dist_name == pkg_dist][0]
                 # the conda 4.4 API uses a single `link_prefs` kwarg
                 # whereas conda 4.3 used `index` and `link_dists` kwargs
-                pfe = ProgressiveFetchExtract(link_prefs=(index[pkg],))
+                pfe = ProgressiveFetchExtract(link_prefs=(index[pkg_record],))
             except TypeError:
                 # TypeError: __init__() got an unexpected keyword argument 'link_prefs'
                 pfe = ProgressiveFetchExtract(link_dists=[pkg], index=index)
@@ -267,9 +289,28 @@ def get_upstream_pins(m, actions, env):
                 if os.path.isfile(_loc):
                     pkg_loc = _loc
                     break
+        pkg_files[pkg] = pkg_loc, pkg_dist
 
-        specs = _read_specs_from_package(pkg_loc, pkg_dist)
+    return pkg_files
 
+
+def get_upstream_pins(m, actions, env):
+    """Download packages from specs, then inspect each downloaded package for additional
+    downstream dependency specs.  Return these additional specs."""
+
+    env_specs = m.meta.get('requirements', {}).get(env, [])
+    explicit_specs = [req.split(' ')[0] for req in env_specs] if env_specs else []
+    linked_packages = actions.get('LINK', [])
+    linked_packages = [pkg for pkg in linked_packages if pkg.name in explicit_specs]
+
+    pkg_locs_and_dists = execute_download_actions(m, actions, env=env,
+                                                  package_subset=linked_packages)
+
+    ignore_list = utils.ensure_list(m.get_value('build/ignore_run_exports'))
+
+    additional_specs = {}
+    for (loc, dist) in pkg_locs_and_dists.values():
+        specs = _read_specs_from_package(loc, dist)
         additional_specs = utils.merge_dicts_of_lists(additional_specs,
                                                       _filter_run_exports(specs, ignore_list))
     return additional_specs
@@ -329,6 +370,80 @@ def add_upstream_pins(m, permit_unsatisfiable_variants, exclude_pattern):
     return build_unsat, host_unsat
 
 
+class _Constraint(object):
+    def __init__(self, lower_bound=None, upper_bound=None, exact=None):
+        self.lower_bound = VersionOrder(lower_bound) if lower_bound else None
+        self.upper_bound = VersionOrder(upper_bound) if upper_bound else None
+        self.exact = exact
+
+    def __str__(self):
+        pin = ""
+        if self.exact:
+            pin = self.exact
+        elif self.lower_bound:
+            pin = ">=" + self.lower_bound.norm_version + (
+                ',<' + self.upper_bound.norm_version) if self.upper_bound else ""
+        elif self.upper_bound:
+            pin = "<" + self.upper_bound.norm_version
+        return pin
+
+
+def _minimize_constraints(one, two):
+    if one.lower_bound and two.lower_bound:
+        lower_bound = max(VersionOrder(one.lower_bound), VersionOrder(two.lower_bound))
+    else:
+        lower_bound = one.lower_bound if one.lower_bound else two.lower_bound
+    if one.upper_bound and two.upper_bound:
+        upper_bound = min(VersionOrder(one.upper_bound), VersionOrder(two.upper_bound))
+    else:
+        upper_bound = one.upper_bound if one.upper_bound else two.upper_bound
+    if one.exact == two.exact or any(x.exact is None for x in (one, two)):
+        exact = one.exact if one.exact else two.exact
+    elif one.exact != two.exact:
+        raise ValueError("Incompatible exact pins: {} and {}".format(one, two))
+    return _Constraint(lower_bound=lower_bound, upper_bound=upper_bound, exact=exact)
+
+
+def _translate_equal_or_star_to_bounded(spec_text):
+    version = re.search(r'(?:==)([\w\.]+)|([\w\.]+)(?:\*)', spec_text)
+    if version:
+        version = version.group(1) if version.group(1) else version.group(2)
+        spec_text = version.rstrip('.')
+        spec_text = utils.apply_pin_expressions(spec_text, max_pin='x.x.x.x.x.x.x.x.x.x.x')
+    return spec_text
+
+
+def _simplify_to_tightest_constraint(metadata):
+    requirements = metadata.meta.get('requirements', {})
+    default_constraint = _Constraint(None, None, None)
+    # collect deps on a per-section basis
+    for section in 'build', 'host', 'run':
+        deps = requirements.get(section, [])
+        deps_dict = defaultdict(_Constraint)
+        for dep in deps:
+            spec_parts = utils.ensure_valid_spec(dep).split()
+            name = spec_parts[0]
+            if len(spec_parts) == 1:
+                constraint = _Constraint(None, None, None)
+            elif len(spec_parts) == 3:
+                constraint = _Constraint(None, None, " ".join(spec_parts[1:]))
+            else:
+                version_spec = _translate_equal_or_star_to_bounded(spec_parts[1])
+                lower_bound = re.search(r'>=?(.*?(?=,|\Z))', version_spec)
+                if lower_bound:
+                    lower_bound = lower_bound.group(1)
+                upper_bound = re.search(r'<=?(.*?(?=,|\Z))', version_spec)
+                if upper_bound:
+                    upper_bound = upper_bound.group(1)
+                constraint = _Constraint(lower_bound, upper_bound, None)
+            deps_dict[name] = _minimize_constraints(deps_dict.get(name, default_constraint),
+                                                    constraint)
+        requirements[section] = [' '.join((spec, str(constraint))).rstrip() for spec, constraint in
+                                          deps_dict.items()]
+    if requirements:
+        metadata.meta['requirements'] = requirements
+
+
 def finalize_metadata(m, permit_unsatisfiable_variants=False):
     """Fully render a recipe.  Fill in versions for build/host dependencies."""
     if m.skip():
@@ -451,6 +566,8 @@ def finalize_metadata(m, permit_unsatisfiable_variants=False):
         if not rendered_metadata.meta.get('build'):
             rendered_metadata.meta['build'] = {}
 
+        _simplify_to_tightest_constraint(rendered_metadata)
+
         if build_unsat or host_unsat:
             rendered_metadata.final = False
             log = utils.get_logger(__name__)
@@ -650,14 +767,6 @@ def render_recipe(recipe_path, config, no_download_source=False, variants=None,
             m.config.variant = m.config.variants[0]
         rendered_metadata = [(m, False, False), ]
     else:
-        index, index_ts = get_build_index(m.config.build_subdir,
-                                          bldpkgs_dir=m.config.bldpkgs_dir,
-                                          output_folder=m.config.output_folder,
-                                          channel_urls=m.config.channel_urls,
-                                          omit_defaults=m.config.override_channels,
-                                          debug=m.config.debug, verbose=m.config.verbose,
-                                          locking=m.config.locking, timeout=m.config.timeout)
-
         # merge any passed-in variants with any files found
         variants = get_package_variants(m, variants=variants)
 
