@@ -626,6 +626,16 @@ unxz is required to unarchive .xz source files.
         check_call_env([unxz, '-f', '-k', tarball])
         tarball = tarball[:-3]
     t = tarfile.open(tarball, mode)
+    members = t.getmembers()
+    for i, member in enumerate(members, 0):
+        if os.path.isabs(member.name):
+            member.name = os.path.relpath(member.name, '/')
+        if not os.path.realpath(member.name).startswith(os.getcwd()):
+            member.name = member.name.replace("../", "")
+        if not os.path.realpath(member.name).startswith(os.getcwd()):
+            sys.exit("tarball contains unsafe path: " + member.name)
+        members[i] = member
+
     if not PY3:
         t.extractall(path=dir_path.encode(codec))
     else:
@@ -1390,7 +1400,7 @@ def merge_or_update_dict(base, new, path, merge, raise_on_clobber=False):
                 if base_value and base_value != value:
                     base_value.extend(value)
                 try:
-                    base[key] = list(set(base_value))
+                    base[key] = list(base_value)
                 except TypeError:
                     base[key] = base_value
             else:
