@@ -27,13 +27,11 @@ DEFAULT_VARIANTS = {
     'lua': '5',
     'r_base': '3.4',
     'cpu_optimization_target': 'nocona',
-    'pin_run_as_build': {'python': {'min_pin': 'x.x', 'max_pin': 'x.x'},
-                         'r-base': {'min_pin': 'x.x.x', 'max_pin': 'x.x.x'}},
+    'pin_run_as_build': OrderedDict(python=OrderedDict(min_pin='x.x', max_pin='x.x'),
+                                    r_base=OrderedDict(min_pin='x.x.x', max_pin='x.x.x')),
     'ignore_version': [],
     'ignore_build_only_deps': ['python'],
     'extend_keys': ['pin_run_as_build', 'ignore_version', 'ignore_build_only_deps'],
-    # We use MRAN here because they take snapshots which improves reproducibility.
-    'cran_mirror': 'https://mran.microsoft.com/snapshot/2018-01-01',
 }
 
 # map python version to default compiler on windows, to match upstream python
@@ -335,9 +333,10 @@ def _get_zip_dict_of_lists(combined_variant, list_of_strings):
         length = len(ensure_list(combined_variant[used_keys[0]]))
         for key in used_keys:
             if not len(ensure_list(combined_variant[key])) == length:
-                raise ValueError("zip field {} length does not match zip field {} length.  All zip "
-                                 "fields within a group must be the same length."
-                                 .format(used_keys[0], key))
+                raise ValueError("zip field {} ({}) length does not match zip field {} ({}) "
+                                 "length.  All zip fields within a group must be the same length."
+                                 .format(used_keys[0], combined_variant[used_keys[0]],
+                                         key, combined_variant[key]))
         values = list(zip(*[ensure_list(combined_variant[key]) for key in used_keys]))
         values = ['#'.join(value) for value in values]
         out = {dict_key: values}
@@ -421,7 +420,7 @@ def list_of_dicts_to_dict_of_lists(list_of_dicts):
     """
     if not list_of_dicts:
         return
-    squished = {}
+    squished = OrderedDict()
     all_zip_keys = set()
     groups = None
     zip_key_groups = (list_of_dicts[0]['zip_keys'] if 'zip_keys' in list_of_dicts[0] and
@@ -440,7 +439,7 @@ def list_of_dicts_to_dict_of_lists(list_of_dicts):
             if k == 'zip_keys':
                 continue
             if hasattr(v, 'keys'):
-                existing_value = squished.get(k, {})
+                existing_value = squished.get(k, OrderedDict())
                 existing_value.update(v)
                 squished[k] = existing_value
             elif isinstance(v, list):
@@ -522,7 +521,7 @@ def find_used_variables_in_text(variant, recipe_text):
     used_variables = set()
     for v in variant:
         variant_regex = r"(\{\{\s*(?:pin_\w+\(\s*['\"]+)?%s(?:[^\}]+)\}\})" % v
-        selector_regex = r"\#?\s\[.*?((?<![_\w\d])%s)[=\s<>!\]]" % v
+        selector_regex = r"\#?\s\[[^\]]*?((?<![_\w\d])%s)[=\s<>!\]]" % v
         conditional_regex = r"(.*?\{%\s*(?:el)?if\s*" + v + r"\s*(?:.*?)?%\})"
         requirement_regex = r"(\-\s+%s(?:\s+[\[#]|$))" % v.replace('_', '[-_]')
         all_res = [variant_regex, selector_regex, conditional_regex, requirement_regex]
