@@ -202,9 +202,6 @@ def msvc_env_cmd(bits, config, override=None):
 
 
 def _write_bat_activation_text(file_handle, m):
-    file_handle.write('call "{conda_root}\\activate.bat" "{prefix}"\n'.format(
-        conda_root=root_script_dir,
-        prefix=m.config.build_prefix))
     if m.is_cross:
         # HACK: we need both build and host envs "active" - i.e. on PATH,
         #     and with their activate.d scripts sourced. Conda only
@@ -225,15 +222,19 @@ def _write_bat_activation_text(file_handle, m):
             if not isdir(dirname(history_file)):
                 os.makedirs(dirname(history_file))
             open(history_file, 'a').close()
-        # removing this placeholder should make conda double-activate with conda 4.3
-        file_handle.write('set "PATH=%PATH:CONDA_PATH_PLACEHOLDER;=%"\n')
-        file_handle.write('set CONDA_MAX_SHLVL=2\n')
         file_handle.write('call "{conda_root}\\activate.bat" "{prefix}"\n'.format(
             conda_root=root_script_dir,
             prefix=m.config.host_prefix))
+        # removing this placeholder should make conda double-activate with conda 4.3
+        file_handle.write('set "PATH=%PATH:CONDA_PATH_PLACEHOLDER;=%"\n')
+        file_handle.write('set CONDA_MAX_SHLVL=2\n')
+    # Write build prefix activation AFTER host prefix, so that its executables come first
+    file_handle.write('call "{conda_root}\\activate.bat" "{prefix}"\n'.format(
+        conda_root=root_script_dir,
+        prefix=m.config.build_prefix))
 
 
-def build(m, bld_bat):
+def build(m, bld_bat, stats):
     with path_prepended(m.config.build_prefix):
         with path_prepended(m.config.host_prefix):
             env = environ.get_dict(config=m.config, m=m)
@@ -270,6 +271,6 @@ def build(m, bld_bat):
             fo.write(data)
 
         cmd = ['cmd.exe', '/c', 'bld.bat']
-        check_call_env(cmd, cwd=src_dir)
+        check_call_env(cmd, cwd=src_dir, stats=stats)
 
     fix_staged_scripts(join(m.config.host_prefix, 'Scripts'), config=m.config)
